-- GLOBALS
local MineOMatic = CreateFrame("Frame", "MineOMatic")
local coffeeId = 118897 -- Miner's Coffee
local pickId = 118903 -- Preserved Mining Pick
local bagScanned

local isInGarrison = C_Garrison.IsOnGarrisonMap
local isInPartyGarrison = C_Garrison.IsUsingPartyGarrison

-- EventHandler
function MineOMatic_EventHandler(self, event, arg1, arg2, ...)
    if event == "ADDON_LOADED" and arg1 == "Mine-O-Matic" then
        SlashCmdList["MINEOMATIC"] = MineOMatic_SlashCommand
        if not MineOMaticSettings then
            MineOMaticSettings = {}
            MineOMaticSettings.maxCoffeeStack = 4
            MineOMaticSettings.maxPickStack = 4
            MineOMaticSettings.runSilent = false
        end
        -- update older versions of addon to use new variable array
        if maxStackSize then
            MineOMaticSettings.maxCoffeeStack = maxStackSize
            MineOMaticSettings.maxPickStack = maxStackSize
        end
        if runSilent then
            MineOMaticSettings.runSilent = runSilent
        end
        if not MineOMaticSettings.runSilent then
            ChatFrame1:AddMessage("|cFF33FF99Mine-O-Matic|r is ready.")
        end
    end
    if event == "ITEM_PUSH" then
        if isInGarrison() and not isInPartyGarrison() then
            bagScanned = false
            C_Timer.After(2.5, MineOMatic_ScanBags)
        end
    end
end

-- Register Events
MineOMatic:RegisterEvent("ADDON_LOADED")
MineOMatic:RegisterEvent("ITEM_PUSH")

-- SetScript
MineOMatic:SetScript("OnEvent", MineOMatic_EventHandler)

-- Slash Commands
SLASH_MINEOMATIC1, SLASH_MINEOMATIC2, SLASH_MINEOMATIC3 = "/mine-o-matic", "/mineomatic", "/mm"

-- Slash Command Handler
function MineOMatic_SlashCommand(msg, editbox)
    if msg ~= nil then
        if msg == "0" or msg == "1" or msg == "2" or msg == "3" or msg == "4" then
            MineOMaticSettings.maxCoffeeStack = tonumber(msg)
            MineOMaticSettings.maxPickStack = tonumber(msg)
            ChatFrame1:AddMessage("|cFF33FF99Mine-O-Matic|r power level " .. MineOMaticSettings.maxCoffeeStack .. ".")
        elseif msg == "coffee 0" then
            MineOMaticSettings.maxCoffeeStack = 0
            ChatFrame1:AddMessage("|cFF33FF99Mine-O-Matic|r power level " .. MineOMaticSettings.maxCoffeeStack .. " for coffee.")
        elseif msg == "coffee 1" then
            MineOMaticSettings.maxCoffeeStack = 1
            ChatFrame1:AddMessage("|cFF33FF99Mine-O-Matic|r power level " .. MineOMaticSettings.maxCoffeeStack .. " for coffee.")
        elseif msg == "coffee 2" then
            MineOMaticSettings.maxCoffeeStack = 2
            ChatFrame1:AddMessage("|cFF33FF99Mine-O-Matic|r power level " .. MineOMaticSettings.maxCoffeeStack .. " for coffee.")
        elseif msg == "coffee 3" then
            MineOMaticSettings.maxCoffeeStack = 3
            ChatFrame1:AddMessage("|cFF33FF99Mine-O-Matic|r power level " .. MineOMaticSettings.maxCoffeeStack .. " for coffee.")
        elseif msg == "coffee 4" then
            MineOMaticSettings.maxCoffeeStack = 4
            ChatFrame1:AddMessage("|cFF33FF99Mine-O-Matic|r power level " .. MineOMaticSettings.maxCoffeeStack .. " for coffee.")
        elseif msg == "pick 0" then
            MineOMaticSettings.maxPickStack = 0
            ChatFrame1:AddMessage("|cFF33FF99Mine-O-Matic|r power level " .. MineOMaticSettings.maxPickStack .. " for picks.")
        elseif msg == "pick 1" then
            MineOMaticSettings.maxPickStack = 1
            ChatFrame1:AddMessage("|cFF33FF99Mine-O-Matic|r power level " .. MineOMaticSettings.maxPickStack .. " for picks.")
        elseif msg == "pick 2" then
            MineOMaticSettings.maxPickStack = 2
            ChatFrame1:AddMessage("|cFF33FF99Mine-O-Matic|r power level " .. MineOMaticSettings.maxPickStack .. " for picks.")
        elseif msg == "pick 3" then
            MineOMaticSettings.maxPickStack = 3
            ChatFrame1:AddMessage("|cFF33FF99Mine-O-Matic|r power level " .. MineOMaticSettings.maxPickStack .. " for picks.")
        elseif msg == "pick 4" then
            MineOMaticSettings.maxPickStack = 4
            ChatFrame1:AddMessage("|cFF33FF99Mine-O-Matic|r power level " .. MineOMaticSettings.maxPickStack .. " for picks.")
        elseif msg == "now" or msg == "force" then
            bagScanned = false
            MineOMatic_ScanBags()
        elseif msg == "silent" then
            MineOMaticSettings.runSilent = true
            ChatFrame1:AddMessage("|cFF33FF99Mine-O-Matic|r has been put into silent mode.")
        elseif msg == "notify" or msg == "notsilent" then
            MineOMaticSettings.runSilent = false
            ChatFrame1:AddMessage("|cFF33FF99Mine-O-Matic|r has been taken off of silent mode.")
        elseif msg == "reset" then
            MineOMaticSettings.maxCoffeeStack = 4
            MineOMaticSettings.maxPickStack = 4
            MineOMaticSettings.runSilent = false
            ChatFrame1:AddMessage("|cFF33FF99Mine-O-Matic|r has been reset to power level 4.")
        elseif msg == "report" then
            local inGarrison, inPartyGarrison
            local factionGroup, factionName = UnitFactionGroup("player")
            if isInGarrison() then inGarrison = "true" else inGarrison = "false" end
            if isInPartyGarrison() then inPartyGarrison = "true" else inPartyGarrison = "false" end
            ChatFrame1:AddMessage("|cFF33FF99Mine-O-Matic|r report:")
            ChatFrame1:AddMessage("    Version: " .. GetAddOnMetadata("Mine-O-Matic", "Version"))
            ChatFrame1:AddMessage("    Locale: " .. GetLocale())
            ChatFrame1:AddMessage("    Zone: " .. GetZoneText())
            ChatFrame1:AddMessage("    Subzone: " .. GetSubZoneText())
            ChatFrame1:AddMessage("    Faction: " .. factionGroup)
            ChatFrame1:AddMessage("    Garrison: " .. inGarrison)
            ChatFrame1:AddMessage("    Party Garrison: " .. inPartyGarrison)
        elseif msg == "help" then
            ChatFrame1:AddMessage("Basic Usage: |cFF33FF99/mine-o-matic|r [command]")
            ChatFrame1:AddMessage("   |cFF33FF99/mine-o-matic 4|r    Default setting, quantities over 4 are destroyed.")
            ChatFrame1:AddMessage("   |cFF33FF99/mine-o-matic 3|r    Quantities over 3 are destroyed.")
            ChatFrame1:AddMessage("   |cFF33FF99/mine-o-matic 2|r    Quantities over 2 are destroyed.")
            ChatFrame1:AddMessage("   |cFF33FF99/mine-o-matic 1|r    Quantities over 1 are destroyed.")
            ChatFrame1:AddMessage("   |cFF33FF99/mine-o-matic 0|r    All items are destroyed.")
            ChatFrame1:AddMessage("Other commands:")
            ChatFrame1:AddMessage("   |cFF33FF99/mine-o-matic silent|r    Turn off text notification.")
            ChatFrame1:AddMessage("   |cFF33FF99/mine-o-matic notify|r    Turn on text notification.")
            ChatFrame1:AddMessage("   |cFF33FF99/mine-o-matic reset|r    Reset to defaults, 4 items and notification on.")
            ChatFrame1:AddMessage("   |cFF33FF99/mine-o-matic now|r    Manually run the bag scan.")
            ChatFrame1:AddMessage("   |cFF33FF99/mine-o-matic help|r    Display this help text.")
        else
            MineOMatic_CmdUsage()
        end
    else
        MineOMatic_CmdUsage()
    end
end

function MineOMatic_CmdUsage()
    local currentMode
    if MineOMaticSettings.maxCoffeeStack == MineOMaticSettings.maxPickStack then
        currentMode = "power level " .. MineOMaticSettings.maxCoffeeStack
    else
        currentMode = "Coffee [" .. MineOMaticSettings.maxCoffeeStack .. "], Picks [" .. MineOMaticSettings.maxPickStack .. "]"
    end
    if MineOMaticSettings.runSilent then 
        currentMode = currentMode .. " in |cFF33FF99silent mode|r"
    end
    ChatFrame1:AddMessage("|cFF33FF99Mine-O-Matic|r is currently set to: " .. currentMode .. ".")
    ChatFrame1:AddMessage("For complete list of options, type: |cFF33FF99/mine-o-matic help|r")
end

function MineOMatic_ScanBags()
    local itemId, itemLink
    local bagId, slotId
    local stackSize
    -- keep bag scans to a minimum
    if bagScanned == true then return end
    bagScanned = true
    -- scan all the bags slots
    for bagId = 0, NUM_BAG_SLOTS do
        for slotId = 1, GetContainerNumSlots(bagId) do
            itemLink = GetContainerItemLink(bagId, slotId)
            if itemLink ~= nil then
                _, _, itemId = string.find(itemLink, "item:(%d+)")
                itemId = tonumber(itemId)
                if itemId == coffeeId then
                    ClearCursor()
                    _, stackSize = GetContainerItemInfo(bagId, slotId)
                    stackSize = tonumber(stackSize)
                    if MineOMaticSettings.maxCoffeeStack == 0 then
                        PickupContainerItem(bagId, slotId)
                        DeleteCursorItem()
                        if not MineOMaticSettings.runSilent then
                            ChatFrame1:AddMessage("|cFF33FF99Mine-O-Matic|r: " .. itemLink .. "|cFF339933x" .. stackSize .. "|r destroyed.")
                        end
                    elseif stackSize > MineOMaticSettings.maxCoffeeStack then
                        SplitContainerItem(bagId, slotId, stackSize - MineOMaticSettings.maxCoffeeStack)
                        DeleteCursorItem()
                        if not MineOMaticSettings.runSilent then
                            ChatFrame1:AddMessage("|cFF33FF99Mine-O-Matic|r: " .. itemLink .. "|cFF339933x" .. stackSize - MineOMaticSettings.maxCoffeeStack .. "|r destroyed.")
                        end
                    end
                end
                if itemId == pickId then
                    ClearCursor()
                    _, stackSize = GetContainerItemInfo(bagId, slotId)
                    stackSize = tonumber(stackSize)
                    if MineOMaticSettings.maxPickStack == 0 then
                        PickupContainerItem(bagId, slotId)
                        DeleteCursorItem()
                        if not MineOMaticSettings.runSilent then
                            ChatFrame1:AddMessage("|cFF33FF99Mine-O-Matic|r: " .. itemLink .. "|cFF339933x" .. stackSize .. "|r destroyed.")
                        end
                    elseif stackSize > MineOMaticSettings.maxPickStack then
                        SplitContainerItem(bagId, slotId, stackSize - MineOMaticSettings.maxPickStack)
                        DeleteCursorItem()
                        if not MineOMaticSettings.runSilent then
                            ChatFrame1:AddMessage("|cFF33FF99Mine-O-Matic|r: " .. itemLink .. "|cFF339933x" .. stackSize - MineOMaticSettings.maxPickStack .. "|r destroyed.")
                        end
                    end
                end
            end
        end
    end
end
